#include <sstream>
#include <stack>
#include "testlib.h"

using namespace std;

typedef pair<int, int> ii;

vector<vector<int>> G (2e3+15);
vector<int> visited (2e3+15);

void dfs(int u, int p) {
    if (visited[u]) { return; }
    visited[u] = true;
    for (int v : G[u]) if (v != p) {
        dfs(v, u);
    }
}

int main(int argc, char* argv[]) {
    registerValidation(argc, argv);
    int N = inf.readInt(1, 1e3, "N");
    inf.readEoln();

    while (N--) {
        int L = inf.readInt(1, 500, "L");
        inf.readEoln();

        set<set<ii>> segs;
        for (int i = 0; i < L; i++) {
            set<ii> segment;

            int Ax = inf.readInt(-1e5, 1e5, "Ax");
            inf.readSpace();
            int Ay = inf.readInt(-1e5, 1e5, "Ay");
            inf.readSpace();
            segment.insert(ii(Ax, Ay));

            int Bx = inf.readInt(-1e5, 1e5, "Bx");
            inf.readSpace();
            int By = inf.readInt(-1e5, 1e5, "By");
            inf.readEoln();
            segment.insert(ii(Bx, By));

            if (segment.size() != 2) {
                quitf(_fail, "Duplicate point: (%d, %d)", Ax, Ay);
            }

            if (!segs.count(segment)) {
                segs.insert(segment);
            } else {
                quitf(_fail, "Duplicate segment: (%d, %d), (%d, %d)", Ax, Ay, Bx, By);
            }
        }

        fill(visited.begin(), visited.end(), 0);
        for (int i = 0; i < G.size(); i++) {
            G[i].clear();
        }

        map<ii, int> point_to_node;
        int c = 0;

        for (auto seg : segs) {
            ii p1 = *(seg.begin());
            if (!point_to_node.count(p1)) {
                point_to_node[p1] = c++;
            }
            int n1 = point_to_node[p1];
            ii p2 = *(++seg.begin());
            if (!point_to_node.count(p2)) {
                point_to_node[p2] = c++;
            }
            int n2 = point_to_node[p2];

            G[n1].push_back(n2);
            G[n2].push_back(n1);
        }

        dfs(0, 0);

        for (int i = 0; i < c; i++) if (!visited[i]) {
            quitf(_fail, "Not connected");
        }
    }

    inf.readEof();
}
